/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.dubbo.config.integration.single.exportprovider;

import org.apache.dubbo.common.URL;
import org.apache.dubbo.common.extension.Activate;
import org.apache.dubbo.registry.integration.InterfaceCompatibleRegistryProtocol;
import org.apache.dubbo.registry.integration.RegistryProtocol;
import org.apache.dubbo.registry.integration.RegistryProtocolListener;
import org.apache.dubbo.rpc.Exporter;
import org.apache.dubbo.rpc.cluster.ClusterInvoker;

import static org.apache.dubbo.config.integration.Constants.SINGLE_CONFIG_CENTER_EXPORT_PROVIDER;

/**
 * The {@link RegistryProtocolListener} for {@link SingleRegistryCenterExportProviderService}
 */
@Activate(order = 100, value = SINGLE_CONFIG_CENTER_EXPORT_PROVIDER)
public class SingleRegistryCenterExportProviderRegistryProtocolListener implements RegistryProtocolListener {

    private boolean exported = false;

    /**
     * {@inheritDoc}
     */
    @Override
    public void onExport(RegistryProtocol registryProtocol, Exporter<?> exporter) {
        if (registryProtocol instanceof InterfaceCompatibleRegistryProtocol
            && exporter != null
            && exporter.getInvoker() != null
            && exporter.getInvoker().getInterface().equals(SingleRegistryCenterExportProviderService.class)) {
            this.exported = true;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void onRefer(RegistryProtocol registryProtocol, ClusterInvoker<?> invoker, URL url, URL registryURL) {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void onDestroy() {
    }

    /**
     * Returns if this exporter is exported.
     */
    public boolean isExported() {
        return exported;
    }
}
